<?php
/**
 * Restore order from pending status when Adyen Manual Review expires
 */

namespace FiloBlu\OrderStatus\Controller\Adminhtml\Order;

use Magento\Sales\Api\OrderRepositoryInterface;
use Magento\Framework\App\Config\ScopeConfigInterface;
use Magento\Store\Model\ScopeInterface;

class Restore extends \Magento\Backend\App\Action {
    
    const ADYEN_CREDIT_CARD_PAYMENT_NAME = ['adyen_cc', 'adyen_oneclick'];
    const ADYEN_PAYMENT_REVIEW_PENDING_ORDER_STATUS_CONFIG_PATH = 'payment/adyen_abstract/fraud_manual_review_status';
    const ADYEN_PAYMENT_REVIEW_ACCEPTED_ORDER_STATUS_CONFIG_PATH = 'payment/adyen_abstract/fraud_manual_review_accept_status';
    
    /**
     * @var OrderRepositoryInterface
     */
    protected $orderRepository;
    
    protected $_config;

    /**
     * Constructor
     *
     * @param \Magento\Backend\App\Action\Context $context
     * @param \Magento\Framework\View\Result\PageFactory $resultPageFactory
     */
    public function __construct(
            \Magento\Backend\App\Action\Context $context, 
            \Magento\Framework\View\Result\PageFactory $resultPageFactory, 
            OrderRepositoryInterface $orderRepository,
            ScopeConfigInterface $config
    ) {
        parent::__construct($context);
        $this->resultPageFactory = $resultPageFactory;
        $this->orderRepository = $orderRepository;
        $this->_config = $config;
    }

    /**
     * Restore order
     *
     * @return \Magento\Backend\Model\View\Result\Redirect
     */
    public function execute() {
        $resultRedirect = $this->resultRedirectFactory->create();
        $order = $this->_initOrder();
        if ($order) {
            try {
                if (!$this->canRestore($order)) {
                    throw new \Magento\Framework\Exception\LocalizedException(__('Can\'t restore order.'));
                }
                $accepted_status = $this->_config->getValue(self::ADYEN_PAYMENT_REVIEW_ACCEPTED_ORDER_STATUS_CONFIG_PATH, ScopeInterface::SCOPE_STORE);
                $order->setState(\Magento\Sales\Model\Order::STATE_PROCESSING);
                $order->addStatusHistoryComment('Order restored from pending manual review via mod2-order-status restore button', $accepted_status);
                $this->orderRepository->save($order);
                $this->messageManager->addSuccess(__('You restored the order from pending manual review status.'));
            } catch (\Magento\Framework\Exception\LocalizedException $e) {
                $this->messageManager->addError($e->getMessage());
            } catch (\Exception $e) {
                $this->messageManager->addError($e->getMessage());
                $this->messageManager->addError(__('The order was not pending manual review.'));
            }
            $resultRedirect->setPath('sales/order/view', ['order_id' => $order->getId()]);
            return $resultRedirect;
        }
        $resultRedirect->setPath('sales/*/');
        return $resultRedirect;
    }


//    protected function _isAllowed()
//    {
//        return $this->_authorization->isAllowed('FiloBlu_OrderStatus');
//    }
    
    /**
     * Initialize order model instance
     *
     * @return \Magento\Sales\Api\Data\OrderInterface|false
     */
    protected function _initOrder()
    {
        $id = $this->getRequest()->getParam('order_id');
        try {
            $order = $this->orderRepository->get($id);
        } catch (NoSuchEntityException $e) {
            $this->messageManager->addError(__('This order no longer exists.'));
            $this->_actionFlag->set('', self::FLAG_NO_DISPATCH, true);
            return false;
        } catch (InputException $e) {
            $this->messageManager->addError(__('This order no longer exists.'));
            $this->_actionFlag->set('', self::FLAG_NO_DISPATCH, true);
            return false;
        }
        return $order;
    }
    
    protected function canRestore($order) {
        try {
            $method = $order->getPayment()->getMethod();
        } catch (\Exception $e) {
            $method = NULL;
        }
        
        return in_array($method, self::ADYEN_CREDIT_CARD_PAYMENT_NAME)
                && 
                $order->getStatus() == $this->_config->getValue(self::ADYEN_PAYMENT_REVIEW_PENDING_ORDER_STATUS_CONFIG_PATH, ScopeInterface::SCOPE_STORE);
    }
}
